/*
*  Name:      requeriment_verifier.cc
*  Author:    Rafael Jesus Alcantara Perez
*  Summary:   UESQLC requirement verifier for ANSI/ISO eSQL 92
*  Date:      $Date: 1999/12/02 02:11:44 $
*  Revision:  $Revision: 1.9 $
*
*  Copyright (C) 1996-1999  Rafael Jesus Alcantara Perez <ralcan@vnet.es>
*
*  This program is free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  This program is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  You should have received a copy of the GNU General Public License
*  along with this program; if not, write to the Free Software
*  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
*  MA 02111-1307, USA.
*/

#include <cctype>
#include "requirement_verifier.hh"


//
//  C O N S T R U C T O R S
//

TRequirementVerifier::TRequirementVerifier (void)                :
  mpcl::TBaseRequirementVerifier<TStackValue*> (eFULL_SQL_LEVEL)
{

  addRequirement ( eSQL92_F_5_4_A                                                 ,
                   eSTRICT_SQL_92                                                 ,
                   "<schema name> can not contain an <embedded variable name>"    );
  addRequirement ( eSQL92_F_5_4_B                                                 ,
                   eSTRICT_SQL_92                                                 ,
                   "<authorization identifier> can not contain an <embedded "
                   "variable name>"                                               );
  addRequirement ( eSQL92_SR_5_2_P8                                               ,
                   eFULL_SQL_LEVEL                                                ,
                   "<regular identifier> shall not has more than 128 characters"  );
  addRequirement ( eSQL92_SR_5_2_P9                                               ,
                   eFULL_SQL_LEVEL                                                ,
                   "<delimited identifier body> shall not has more than 128 "
                   "characters"                                                   );
  addRequirement ( eSQL92_SR_5_3_P21                                              ,
                   eFULL_SQL_LEVEL                                                ,
                   "within <datetime literal>, <years value> shall contain four "
                   "digits"                                                       );
  addRequirement ( eSQL92_SR_8_2_P1                                               ,
                   eFULL_SQL_LEVEL                                                ,
                   "the <row value constructor>s shall be of the same degree"     );
  addRequirement ( eSQL92_SR_11_3_P5                                              ,
                   eFULL_SQL_LEVEL                                                ,
                   "a <temporary table declaration> shall contain at least one "
                   "<column definition>"                                          );
  addRequirement ( eSQL92_SR_13_11_P4                                             ,
                   eFULL_SQL_LEVEL                                                ,
                   "a <table definition> shall contain at least one "
                   "<column definition>"                                          );
  addRequirement ( eSQL92_LR_5_2_P1                                               ,
                   eINTERMEDIATE_SQL_LEVEL                                        ,
                   "no <identifier body> shall end in an <underscore>"            );
  addRequirement ( eSQL92_LR_5_2_P2_A                                             ,
                   eENTRY_SQL_LEVEL                                               ,
                   "no <regular identifier> or <delimited identifier body> shall"
                   " contain more than 18 <character representation>s"            );
  addRequirement ( eSQL92_LR_5_2_P2_B                                             ,
                   eENTRY_SQL_LEVEL                                               ,
                   "an <identifier body> shall contain no <simple Latin lower ca"
                   "se letter>"                                                   );
  addRequirement ( eSQL92_LR_6_3_P2_B                                             ,
                   eENTRY_SQL_LEVEL                                               ,
                   "the optional <key word> AS shall not be specified"            );
  addRequirement ( eSQL92_LR_6_3_P2_C                                             ,
                   eENTRY_SQL_LEVEL                                               ,
                   "<derived column list> shall not be specified"                 );
  addRequirement ( eSQL92_LR_6_8_P1                                               ,
                   eINTERMEDIATE_SQL_LEVEL                                        ,
                   "language shall contain no <time precision> or "
                   "<timestamp precision>"                                        );
  addRequirement ( eSQL92_LR_6_8_P2                                               ,
                   eENTRY_SQL_LEVEL                                               ,
                   "language shall not contain any <datetime value function>"     );
  addRequirement ( eSQL92_LR_6_11_P2_A                                            ,
                   eENTRY_SQL_LEVEL                                               ,
                   "a <value expression> shall not be a "
                   "<datetime value expression>"                                  );
  addRequirement ( eSQL92_LR_6_11_P2_B                                            ,
                   eENTRY_SQL_LEVEL                                               ,
                   "a <value expression> shall not be a "
                   "<interval value expression>"                                  );
  addRequirement ( eSQL92_LR_6_11_P2_C                                            ,
                   eENTRY_SQL_LEVEL                                               ,
                   "a <value expression primary> shall not be a "
                   "<case expression>"                                            );
  addRequirement ( eSQL92_LR_6_11_P2_D                                            ,
                   eENTRY_SQL_LEVEL                                               ,
                   "a <value expression primary> shall not be a "
                   "<cast specification>"                                         );
  addRequirement ( eSQL92_LR_8_9_P2                                               ,
                   eENTRY_SQL_LEVEL                                               ,
                   "language shall not contain any <unique predicate>"            );
  addRequirement ( eSQL92_LR_8_10_P1                                              ,
                   eINTERMEDIATE_SQL_LEVEL                                        ,
                   "language shall not contain any <match predicate>"             );
  addRequirement ( eSQL92_LR_11_4_P2_A                                            ,
                   eENTRY_SQL_LEVEL                                               ,
                   "a <column definition> shall not contain a <domain name>"      );
  addRequirement ( eSQL92_LR_11_10_P2                                             ,
                   eENTRY_SQL_LEVEL                                               ,
                   "language shall not contain any <alter table statement>"       );
  addRequirement ( eSQL92_LR_13_1_P1_A                                            ,
                   eINTERMEDIATE_SQL_LEVEL                                        ,
                   "a <declare cursor> shall not specify INSENSITIVE"             );
  addRequirement ( eSQL92_LR_13_1_P2_A                                            ,
                   eENTRY_SQL_LEVEL                                               ,
                   "a <declare cursor> shall not specify SCROLL"                  );
  addRequirement ( eSQL92_LR_13_1_P2_B                                            ,
                   eENTRY_SQL_LEVEL                                               ,
                   "a <cursor specification> shall not contain an "
                   "<updatability clause>"                                        );
  addRequirement ( eSQL92_LR_13_3_P2_A                                            ,
                   eENTRY_SQL_LEVEL                                               ,
                   "a <fetch statement> shall not specify a "
                   "<fetch orientation>"                                          );
  addRequirement ( eSQL92_LR_13_3_P2_B                                            ,
                   eENTRY_SQL_LEVEL                                               ,
                   "a <fetch statement> shall not specify FROM"                   );

}  // TRequirementVerifier()


//
//  S E L E C T O R S
//

bool TRequirementVerifier::verify (long int liREQUERIMENT_IDENTIFIER) const
{

  bool   gSuccess = false;

  switch (liREQUERIMENT_IDENTIFIER)
  {
    case eSQL92_SR_5_2_P8:
    case eSQL92_SR_5_2_P9:
    {
      gSuccess = ( tArgumentArray [0]->yCode.length() <= 128 );
      break;
    }
    case eSQL92_SR_5_3_P21:
    {
      size_t   zOffset = 0;

      zOffset  = tArgumentArray [0]->yCode.find ('-');
      gSuccess = ( zOffset == 3 );
      break;
    }
    case eSQL92_SR_8_2_P1:
    {
      gSuccess = ( tArgumentArray [0]->iItemCount == tArgumentArray [1]->iItemCount );
      break;
    }
    case eSQL92_SR_11_3_P5:
    case eSQL92_SR_13_11_P4:
    {
      gSuccess = ( tArgumentArray [0]->gHasColumnDefinition );
      break;
    }
    case eSQL92_LR_5_2_P1:
    {
      string&   ryCode         = tArgumentArray [0]->yCode;
      size_t    zLastCharacter = ryCode.length() - 1;
      
      gSuccess = ( ryCode [zLastCharacter] != '_' );
      break;
    }
    case eSQL92_LR_5_2_P2_A:
    {
      gSuccess = ( tArgumentArray [0]->yCode.length() <= 18 );
      break;
    }
    case eSQL92_LR_5_2_P2_B:
    {
      string&   ryCode        = tArgumentArray [0]->yCode;
      size_t    zStringLength = ryCode.length();
      
      gSuccess = true;
      for (register size_t J = 0; ( J < zStringLength ) ;J++)
      {
	if ( islower (ryCode [J]) )
	{
	  gSuccess = false;
	  break;
	}
      }
      break;
    }
    case eSQL92_F_5_4_A:
    case eSQL92_F_5_4_B:
    case eSQL92_LR_6_3_P2_B:
    case eSQL92_LR_6_3_P2_C:
    case eSQL92_LR_6_8_P1:
    case eSQL92_LR_6_8_P2:
    case eSQL92_LR_6_11_P2_A:
    case eSQL92_LR_6_11_P2_B:
    case eSQL92_LR_6_11_P2_C:
    case eSQL92_LR_6_11_P2_D:
    case eSQL92_LR_8_9_P2:
    case eSQL92_LR_8_10_P1:
    case eSQL92_LR_11_4_P2_A:
    case eSQL92_LR_11_10_P2:
    case eSQL92_LR_13_1_P1_A:
    case eSQL92_LR_13_1_P2_A:
    case eSQL92_LR_13_1_P2_B:
    case eSQL92_LR_13_3_P2_A:
    case eSQL92_LR_13_3_P2_B:
    {
      break;
    }
  }
  return gSuccess;

}  // verify()
